@@ -1,4 +1,4 @@
-"""genft
+"""sonusai genft
 
 usage: genft [-hvs] (-d MIXDB) [-i MIXID] [-o OUTPUT]
 
@@ -47,12 +47,14 @@ from sonusai import create_file_handler
 from sonusai import initial_log_messages
 from sonusai import logger
 from sonusai import update_console_handler
-from sonusai.mixture import apply_augmentation
-from sonusai.mixture import audio_files_exist
 from sonusai.mixture import build_noise_audio_db
 from sonusai.mixture import build_target_audio_db
+from sonusai.mixture import check_audio_files_exist
 from sonusai.mixture import get_audio_and_truth_t
+from sonusai.mixture import get_feature_frames_in_mixture
+from sonusai.mixture import get_samples_in_mixture
 from sonusai.mixture import get_total_class_count
+from sonusai.mixture import get_transform_frames_in_mixture
 from sonusai.mixture import load_mixdb
 from sonusai.mixture import load_mixid
 from sonusai.mixture import new_mixdb_from_mixid
@@ -106,9 +108,10 @@ def genft(mixdb: dict,
           logging: bool = True,
           show_progress: bool = False,
           progress: tqdm = None,
-          parallel: bool = True,
-          filename: str = None,
-          batch_size: int = None) -> (np.ndarray, np.ndarray, dict):
+          initial_i_sample_offset: int = 0,
+          initial_i_frame_offset: int = 0,
+          initial_o_frame_offset: int = 0,
+          audio_loaded: bool = False) -> (np.ndarray, np.ndarray, dict):
     mixdb_out, total_samples, transform_frames, feature_frames = init(mixdb=mixdb,
                                                                       mixid=mixid,
                                                                       logging=logging)
@@ -116,8 +119,9 @@ def genft(mixdb: dict,
     genft_global['mixdb'] = mixdb_out
     genft_global['feature_frames'] = feature_frames
     genft_global['compute_segsnr'] = compute_segsnr
-    genft_global['noise_audios'] = build_noise_audio_db(mixdb=mixdb_out, show_progress=show_progress)
-    genft_global['target_audios'] = build_target_audio_db(mixdb=mixdb_out, show_progress=show_progress)
+    if not audio_loaded:
+        genft_global['noise_audios'] = build_noise_audio_db(mixdb=mixdb_out, show_progress=show_progress)
+        genft_global['target_audios'] = build_target_audio_db(mixdb=mixdb_out, show_progress=show_progress)
 
     fg = FeatureGenerator(frame_size=mixdb_out['frame_size'],
                           feature_mode=mixdb_out['feature'],
@@ -136,7 +140,10 @@ def genft(mixdb: dict,
         segsnr = to_numpy_array(mp_segsnr, dtype=np.single)
 
     # First pass to get offsets
-    set_mixture_offsets(mixdb_out)
+    set_mixture_offsets(mixdb=mixdb_out,
+                        initial_i_sample_offset=initial_i_sample_offset,
+                        initial_i_frame_offset=initial_i_frame_offset,
+                        initial_o_frame_offset=initial_o_frame_offset)
 
     # Second pass to get feature and truth_f
     progress_needs_close = False
@@ -144,24 +151,18 @@ def genft(mixdb: dict,
         progress = tqdm(total=len(mixdb_out['mixtures']), desc='genft', disable=not show_progress)
         progress_needs_close = True
 
-    if parallel:
-        p_map(process_mixture,
-              mixdb_out['mixtures'],
-              progress=progress,
-              initializer=mp_init,
-              initargs=(mp_feature, mp_truth_f, mp_segsnr))
-    else:
-        mp_init(mp_feature, mp_truth_f, mp_segsnr)
-        for record in mixdb_out['mixtures']:
-            process_mixture(record)
-            progress.update()
+    p_map(_process_mixture,
+          list(range(len(mixdb_out['mixtures']))),
+          progress=progress,
+          initializer=mp_init,
+          initargs=(mp_feature, mp_truth_f, mp_segsnr))
 
     if progress_needs_close:
         progress.close()
 
     mixdb_out['class_count'] = get_total_class_count(mixdb_out)
 
-    duration = total_samples / sonusai.mixture.sample_rate
+    duration = total_samples / sonusai.mixture.SAMPLE_RATE
     if logging:
         logger.info('')
         logger.info(f'Duration: {seconds_to_hms(seconds=duration)}')
@@ -173,7 +174,7 @@ def genft(mixdb: dict,
     return feature, truth_f, segsnr, mixdb_out
 
 
-def process_mixture(record: dict) -> None:
+def _process_mixture(mixid: int) -> None:
     fft = ForwardTransform(N=genft_global['mixdb']['frame_size'] * 4,
                            R=genft_global['mixdb']['frame_size'])
 
@@ -190,8 +191,12 @@ def process_mixture(record: dict) -> None:
 
     segsnr = to_numpy_array(mp_segsnr, dtype=np.single)
 
-    frame_indices = slice(record['i_frame_offset'],
-                          record['i_frame_offset'] + record['samples'] // genft_global['mixdb']['frame_size'])
+    record = genft_global['mixdb']['mixtures'][mixid]
+    first_i_frame_offset = genft_global['mixdb']['mixtures'][0]['i_frame_offset']
+    transform_frames = get_transform_frames_in_mixture(genft_global['mixdb'], mixid)
+
+    frame_indices = slice(record['i_frame_offset'] - first_i_frame_offset,
+                          record['i_frame_offset'] - first_i_frame_offset + transform_frames)
 
     (mixture_td,
      truth_t,
@@ -205,8 +210,8 @@ def process_mixture(record: dict) -> None:
                                                     compute_segsnr=genft_global['compute_segsnr'],
                                                     frame_based_segnsr=True)
 
-    transform_frames = record['samples'] // genft_global['mixdb']['frame_size']
-    feature_frame = record['o_frame_offset']
+    first_o_frame_offset = genft_global['mixdb']['mixtures'][0]['o_frame_offset']
+    feature_frame = record['o_frame_offset'] - first_o_frame_offset
     for transform_frame in range(transform_frames):
         indices = slice(transform_frame * genft_global['mixdb']['frame_size'],
                         (transform_frame + 1) * genft_global['mixdb']['frame_size'])
@@ -247,8 +252,7 @@ def main():
                                                                           mixid=mixid,
                                                                           logging=True)
 
-        if not audio_files_exist(mixdb):
-            raise SystemExit(1)
+        check_audio_files_exist(mixdb)
 
         fg = FeatureGenerator(frame_size=mixdb_out['frame_size'],
                               feature_mode=mixdb_out['feature'],
@@ -272,6 +276,7 @@ def main():
         mixid = grouper(range(len(mixdb_out['mixtures'])), chunk_size)
         mixdb_out['class_count'] = [0] * mixdb_out['num_classes']
 
+        i_sample_offset = 0
         i_frame_offset = 0
         o_frame_offset = 0
         for m in mixid:
@@ -279,7 +284,10 @@ def main():
                                                         mixid=m,
                                                         compute_segsnr=compute_segsnr,
                                                         logging=False,
-                                                        progress=progress)
+                                                        progress=progress,
+                                                        initial_i_sample_offset=i_sample_offset,
+                                                        initial_i_frame_offset=i_frame_offset,
+                                                        initial_o_frame_offset=o_frame_offset)
             o_frames = feature.shape[0]
             if o_frames != truth_f.shape[0]:
                 logger.exception(f'truth_f frames does not match feature frames: {truth_f.shape[0]} != {o_frames}')
@@ -287,7 +295,6 @@ def main():
 
             with h5py.File(output_name, 'a') as f:
                 indices = slice(o_frame_offset, o_frame_offset + o_frames)
-                o_frame_offset += o_frames
 
                 feature_dataset = f['feature']
                 feature_dataset[indices] = feature
@@ -298,20 +305,26 @@ def main():
                 if compute_segsnr:
                     segsnr_dataset = f['segsnr']
                     segsnr_dataset[i_frame_offset:i_frame_offset + segsnr.shape[0]] = segsnr
-                    i_frame_offset += segsnr.shape[0]
 
             for idx, val in enumerate(m):
                 mixdb_out['mixtures'][val] = mixdb_tmp['mixtures'][idx]
             for idx in range(mixdb_out['num_classes']):
                 mixdb_out['class_count'][idx] += mixdb_tmp['class_count'][idx]
 
+            i_sample_offset = mixdb_out['mixtures'][m[-1]]['i_sample_offset']
+            i_sample_offset += get_samples_in_mixture(mixdb_out, m[-1])
+            i_frame_offset = mixdb_out['mixtures'][m[-1]]['i_frame_offset']
+            i_frame_offset += get_transform_frames_in_mixture(mixdb_out, m[-1])
+            o_frame_offset = mixdb_out['mixtures'][m[-1]]['o_frame_offset']
+            o_frame_offset += get_feature_frames_in_mixture(mixdb_out, m[-1])
+
         with h5py.File(output_name, 'a') as f:
             f.attrs['mixdb'] = json.dumps(mixdb_out)
 
         progress.close()
 
         logger.info(f'Wrote {output_name}')
-        duration = total_samples / sonusai.mixture.sample_rate
+        duration = total_samples / sonusai.mixture.SAMPLE_RATE
         logger.info('')
         logger.info(f'Duration: {seconds_to_hms(seconds=duration)}')
         logger.info(f'feature:  {human_readable_size(o_frame_offset * fg.stride * fg.num_bands * 4, 1)}')
