@@ -1,4 +1,4 @@
-"""genmixdb
+"""sonusai genmixdb
 
 usage: genmixdb [-hv] CONFIG...
 
@@ -151,7 +151,9 @@ from sonusai.mixture import get_class_weights_threshold
 from sonusai.mixture import get_config_from_file
 from sonusai.mixture import get_next_noise
 from sonusai.mixture import get_total_class_count
+from sonusai.mixture import load_config
 from sonusai.mixture import read_audio
+from sonusai.mixture import update_truth_settings
 from sonusai.utils import expandvars
 from sonusai.utils import human_readable_size
 from sonusai.utils import p_map
@@ -161,29 +163,15 @@ from sonusai.utils import trim_docstring
 genmixdb_global = dict()
 
 
-def genmixdb(config: dict, logging: bool = True, show_progress: bool = False) -> dict:
-    required_keys = [
-        'class_labels',
-        'class_weights_threshold',
-        'dither',
-        'feature',
-        'frame_size',
-        'noises',
-        'noise_augmentations',
-        'num_classes',
-        'seed',
-        'target_augmentations',
-        'targets',
-        'truth_config',
-        'truth_function',
-        'truth_index',
-        'truth_mode',
-        'truth_reduction_function',
-    ]
-    for key in required_keys:
-        if key not in config.keys():
-            logger.exception(f'Missing {key} in config')
-            raise SonusAIError
+def genmixdb(file: str = None,
+             config: dict = None,
+             logging: bool = True,
+             show_progress: bool = False) -> dict:
+    if (file is None and config is None) or (file is not None and config is not None):
+        raise SonusAIError(f'Must specify either file name or config.')
+
+    if file is not None:
+        config = get_config_from_file(file)
 
     seed(config['seed'])
 
@@ -194,12 +182,9 @@ def genmixdb(config: dict, logging: bool = True, show_progress: bool = False) ->
 
     if logging:
         logger.info('Collecting targets')
-    targets = get_input_files(config['targets'], cfg={'truth_index':    config['truth_index'],
-                                                      'truth_function': config['truth_function'],
-                                                      'truth_config':   config['truth_config']})
+    targets = get_input_files(records=config['targets'], truth_settings=config['truth_settings'])
     if len(targets) == 0:
-        logger.exception('Canceled due to no targets')
-        raise SonusAIError
+        raise SonusAIError('Canceled due to no targets')
 
     if logging:
         logger.debug('List of targets:')
@@ -207,18 +192,18 @@ def genmixdb(config: dict, logging: bool = True, show_progress: bool = False) ->
 
     if logging:
         logger.info('Collecting noises')
-    noises = get_input_files(config['noises'])
+    noises = get_input_files(records=config['noises'])
     if len(noises) == 0:
         if logging:
-            logger.warning(f'Did not find any noises; using default noise: {sonusai.mixture.default_noise}')
-        noises = get_input_files([sonusai.mixture.default_noise])
+            logger.warning(f'Did not find any noises; using default noise: {sonusai.mixture.DEFAULT_NOISE}')
+        noises = get_input_files(records=[sonusai.mixture.DEFAULT_NOISE])
     if logging:
         logger.debug('List of noises:')
         logger.debug(yaml.dump([sub['name'] for sub in noises], default_flow_style=False))
 
     if logging:
         logger.info('Collecting target augmentations')
-    target_augmentations = get_augmentations(config['target_augmentations'], target=True)
+    target_augmentations = get_augmentations(rules=config['target_augmentations'], target=True)
     expanded_target_augmentations = ''
     for augmentation in target_augmentations:
         expanded_target_augmentations += f'- {augmentation}\n'
@@ -246,8 +231,7 @@ def genmixdb(config: dict, logging: bool = True, show_progress: bool = False) ->
     class_weights_threshold = get_class_weights_threshold(config)
 
     if config['truth_mode'] not in ['normal', 'mutex']:
-        logger.exception(f'invalid truth_mode: {config["truth_mode"]}')
-        raise SonusAIError
+        raise SonusAIError(f'invalid truth_mode: {config["truth_mode"]}')
 
     truth_mutex = config['truth_mode'] == 'mutex'
     if truth_mutex:
@@ -265,7 +249,7 @@ def genmixdb(config: dict, logging: bool = True, show_progress: bool = False) ->
     step = fg.step
     decimation = fg.decimation
 
-    transform_frame_ms = float(config['frame_size']) / float(sonusai.mixture.sample_rate / 1000)
+    transform_frame_ms = float(config['frame_size']) / float(sonusai.mixture.SAMPLE_RATE / 1000)
     feature_ms = transform_frame_ms * decimation * stride
     feature_step_ms = transform_frame_ms * decimation * step
     feature_samples = config['frame_size'] * decimation * stride
@@ -274,20 +258,20 @@ def genmixdb(config: dict, logging: bool = True, show_progress: bool = False) ->
     total_duration = 0
     for duration in [sub['duration'] for sub in targets]:
         for augmentation in target_augmentations:
-            length = int(duration * sonusai.mixture.sample_rate)
+            length = int(duration * sonusai.mixture.SAMPLE_RATE)
             if 'tempo' in augmentation:
                 length /= augmentation['tempo']
             if length % feature_step_samples:
                 length += feature_step_samples - int(length % feature_step_samples)
-            total_duration += float(length) / sonusai.mixture.sample_rate
+            total_duration += float(length) / sonusai.mixture.SAMPLE_RATE
     total_duration *= noise_sets
 
-    total_samples = total_duration * sonusai.mixture.sample_rate
+    total_samples = total_duration * sonusai.mixture.SAMPLE_RATE
 
-    mixture_bytes = total_samples * sonusai.mixture.bit_depth / 8
-    truth_t_bytes = total_samples * config['num_classes'] * sonusai.mixture.float_bytes
-    feature_bytes = total_samples / feature_step_samples * stride * num_bands * sonusai.mixture.float_bytes
-    truth_f_bytes = total_samples / feature_step_samples * config['num_classes'] * sonusai.mixture.float_bytes
+    mixture_bytes = total_samples * sonusai.mixture.BIT_DEPTH / 8
+    truth_t_bytes = total_samples * config['num_classes'] * sonusai.mixture.FLOAT_BYTES
+    feature_bytes = total_samples / feature_step_samples * stride * num_bands * sonusai.mixture.FLOAT_BYTES
+    truth_f_bytes = total_samples / feature_step_samples * config['num_classes'] * sonusai.mixture.FLOAT_BYTES
 
     if logging:
         logger.info('')
@@ -305,7 +289,6 @@ def genmixdb(config: dict, logging: bool = True, show_progress: bool = False) ->
         'class_count':              list(),
         'class_labels':             config['class_labels'],
         'class_weights_threshold':  list(class_weights_threshold),
-        'dither':                   config['dither'],
         'feature':                  config['feature'],
         'feature_samples':          feature_samples,
         'feature_step_samples':     feature_step_samples,
@@ -325,13 +308,13 @@ def genmixdb(config: dict, logging: bool = True, show_progress: bool = False) ->
         logger.debug('Reading noise audio')
     raw_noise_audio = list()
     for noise in noises:
-        raw_noise_audio.append(read_audio(name=noise['name'], dither=config['dither']))
+        raw_noise_audio.append(read_audio(name=noise['name']))
 
     if logging:
         logger.debug('Reading target audio')
     raw_target_audio = list()
     for target in targets:
-        raw_target_audio.append(read_audio(name=target['name'], dither=config['dither']))
+        raw_target_audio.append(read_audio(name=target['name']))
 
     # First pass to get indices and offsets
     mixtures = [[] for _ in range(noise_sets)]
@@ -346,22 +329,17 @@ def genmixdb(config: dict, logging: bool = True, show_progress: bool = False) ->
                                                  length_common_denominator=1)
             for target_index, target in enumerate(targets):
                 for target_augmentation_index, target_augmentation in enumerate(target_augmentations):
-                    if not isinstance(target['truth_function'], list):
-                        target['truth_function'] = [target['truth_function']]
+                    if not isinstance(target['truth_settings'], list):
+                        target['truth_settings'] = [target['truth_settings']]
 
-                    if not isinstance(target['truth_index'], list):
-                        target['truth_index'] = [target['truth_index']]
+                    update_truth_settings(target['truth_settings'], config['truth_settings'])
 
-                    if len(target['truth_function']) != len(target['truth_index']):
-                        logger.exception('length of truth_index does not match length of truth_function')
-                        raise SonusAIError
+                    for truth_setting in target['truth_settings']:
+                        if not isinstance(truth_setting['index'], list):
+                            truth_setting['index'] = [truth_setting['index']]
 
-                    for i in range(len(target['truth_function'])):
-                        if not isinstance(target['truth_index'][i], list):
-                            target['truth_index'][i] = [target['truth_index'][i]]
-                        if any(idx > max_class for idx in target['truth_index'][i]):
-                            logger.exception('invalid truth_index')
-                            raise SonusAIError
+                        if any(idx > max_class for idx in truth_setting['index']):
+                            raise SonusAIError('invalid truth index')
 
                     mixtures[n_id][t_id] = {'target_file_index':         target_index,
                                             'noise_file_index':          noise_index,
@@ -390,9 +368,8 @@ def genmixdb(config: dict, logging: bool = True, show_progress: bool = False) ->
         genmixdb_global['augmented_noise_audio'] = apply_augmentation(
             audio_in=raw_noise_audio[mixtures[n_id][0]['noise_file_index']],
             augmentation=noise_augmentations[mixtures[n_id][0]['noise_augmentation_index']],
-            length_common_denominator=1,
-            dither=config['dither'])
-        mixtures[n_id] = p_map(process_mixture, mixtures[n_id], progress=progress)
+            length_common_denominator=1)
+        mixtures[n_id] = p_map(_process_mixture, mixtures[n_id], progress=progress)
 
     progress.close()
 
@@ -401,8 +378,8 @@ def genmixdb(config: dict, logging: bool = True, show_progress: bool = False) ->
     mixdb['class_count'] = get_total_class_count(mixdb)
 
     total_samples = sum([sub['samples'] for sub in mixdb['mixtures']])
-    total_duration = total_samples / sonusai.mixture.sample_rate
-    mixture_bytes = total_samples * sonusai.mixture.bit_depth / 8
+    total_duration = total_samples / sonusai.mixture.SAMPLE_RATE
+    mixture_bytes = total_samples * sonusai.mixture.BIT_DEPTH / 8
     truth_t_bytes = total_samples * config['num_classes'] * 4
     feature_bytes = total_samples / feature_step_samples * stride * num_bands * 4
     truth_f_bytes = total_samples / feature_step_samples * config['num_classes'] * 4
@@ -418,43 +395,35 @@ def genmixdb(config: dict, logging: bool = True, show_progress: bool = False) ->
     return mixdb
 
 
-def get_input_files(records, cfg=None):
-    if cfg is None:
-        cfg = dict()
+def get_input_files(records, truth_settings=None):
+    if truth_settings is None:
+        truth_settings = list()
 
     files = list()
     for record in records:
-        append_input_files(files, record, cfg)
+        append_input_files(files, record, truth_settings)
     return files
 
 
-def append_input_files(files: list, in_record, cfg: dict, tokens=None):
+def append_input_files(files: list, in_record, truth_settings: list, tokens=None):
     if tokens is None:
         tokens = dict()
     in_name = in_record
 
     if isinstance(in_record, dict):
-        if 'target_name' in in_record.keys():
+        if 'target_name' in in_record:
             in_name = in_record['target_name']
         else:
-            logger.exception('Target list contained record without \'target_name\' key')
-            raise SonusAIError
+            raise SonusAIError('Target list contained record without target_name')
 
-        if 'truth_index' in in_record.keys():
-            cfg['truth_index'] = in_record['truth_index']
+        if 'truth_settings' in in_record:
+            truth_settings = in_record['truth_settings']
 
-        if 'truth_function' in in_record.keys():
-            cfg['truth_function'] = in_record['truth_function']
-
-        if 'truth_config' in in_record.keys():
-            cfg['truth_config'] = in_record['truth_config']
-
-    (in_name, new_tokens) = expandvars(in_name)
+    in_name, new_tokens = expandvars(in_name)
     tokens.update(new_tokens)
     names = glob(in_name)
     if not names:
-        logger.exception(f'Could not find {in_name}. Make sure path exists')
-        raise SonusAIError
+        raise SonusAIError(f'Could not find {in_name}. Make sure path exists')
     for name in names:
         ext = splitext(name)[1].lower()
         dir_name = dirname(name)
@@ -463,47 +432,49 @@ def append_input_files(files: list, in_record, cfg: dict, tokens=None):
                 child = file
                 if not isabs(child):
                     child = join(dir_name, child)
-                append_input_files(files, child, cfg, tokens)
+                append_input_files(files, child, truth_settings, tokens)
         else:
             try:
                 if ext == '.txt':
-                    with open(name, mode='r') as txt_file:
+                    with open(file=name, mode='r') as txt_file:
                         for line in txt_file:
                             # strip comments
                             child = line.partition('#')[0]
                             child = child.rstrip()
                             if child:
-                                (child, new_tokens) = expandvars(child)
+                                child, new_tokens = expandvars(child)
                                 tokens.update(new_tokens)
                                 if not isabs(child):
                                     child = join(dir_name, child)
-                                append_input_files(files, child, cfg, tokens)
+                                append_input_files(files, child, truth_settings, tokens)
                 elif ext == '.yml':
                     try:
-                        with open(name, mode='r') as yml_file:
-                            yml_config = yaml.safe_load(yml_file)
+                        yml_config = load_config(name)
 
                         if 'targets' in yml_config:
                             for record in yml_config['targets']:
-                                append_input_files(files, record, cfg, tokens)
+                                append_input_files(files, record, truth_settings, tokens)
                     except Exception as e:
-                        logger.exception(f'Error processing {name}: {e}')
-                        raise SonusAIError
+                        raise SonusAIError(f'Error processing {name}: {e}')
                 else:
                     sox.file_info.validate_input_file(name)
                     duration = sox.file_info.duration(name)
                     for key, value in tokens.items():
                         name = name.replace(value, f'${key}')
-                    entry = {'name': name, 'duration': duration}
-                    entry.update(cfg)
-                    if 'truth_function' in entry.keys() and entry['truth_function'] == 'file':
-                        entry['truth_config']['file'] = splitext(name)[0] + '.h5'
+                    entry = {
+                        'name':     name,
+                        'duration': duration,
+                    }
+                    if len(truth_settings) > 0:
+                        entry['truth_settings'] = truth_settings
+                        for truth_setting in entry['truth_settings']:
+                            if 'function' in truth_setting and truth_setting['function'] == 'file':
+                                truth_setting['config']['file'] = splitext(name)[0] + '.h5'
                     files.append(entry)
             except SonusAIError:
                 raise
             except Exception as e:
-                logger.exception(f'Error processing {name}: {e}')
-                raise SonusAIError
+                raise SonusAIError(f'Error processing {name}: {e}')
 
 
 def get_augmentations(rules: list, target: bool) -> list:
@@ -520,9 +491,8 @@ def expand_augmentations(augmentations: list, rule: dict, target: bool):
     rule = {'eq1' if key == 'eq' else key: value for key, value in rule.items()}
 
     for key in rule:
-        if key not in sonusai.mixture.valid_augmentations:
-            logger.exception(f'Invalid augmentation: {key}')
-            raise SonusAIError
+        if key not in sonusai.mixture.VALID_AUGMENTATIONS:
+            raise SonusAIError(f'Invalid augmentation: {key}')
 
         if key in ['eq1', 'eq2', 'eq3']:
             # eq must be a list of length 3 or a list of length 3 lists
@@ -541,8 +511,7 @@ def expand_augmentations(augmentations: list, rule: dict, target: bool):
                 valid = False
 
             if not valid:
-                logger.exception(f'Invalid augmentation value for {key}: {rule[key]}')
-                raise SonusAIError
+                raise SonusAIError(f'Invalid augmentation value for {key}: {rule[key]}')
 
             if multiple:
                 for value in rule[key]:
@@ -558,20 +527,18 @@ def expand_augmentations(augmentations: list, rule: dict, target: bool):
             if isinstance(rule[key], list):
                 for value in rule[key]:
                     if isinstance(value, list):
-                        logger.exception(f'Invalid augmentation value for {key}: {rule[key]}')
-                        raise SonusAIError
+                        raise SonusAIError(f'Invalid augmentation value for {key}: {rule[key]}')
                     expanded_rule = deepcopy(rule)
                     expanded_rule[key] = deepcopy(value)
                     expand_augmentations(augmentations, expanded_rule, target)
                 return
             elif not isinstance(rule[key], Number):
                 if not rule[key].startswith('rand'):
-                    logger.exception(f'Invalid augmentation value for {key}: {rule[key]}')
-                    raise SonusAIError
+                    raise SonusAIError(f'Invalid augmentation value for {key}: {rule[key]}')
 
     # Set default snr for target if needed
     if target and 'snr' not in rule:
-        rule['snr'] = sonusai.mixture.default_snr
+        rule['snr'] = sonusai.mixture.DEFAULT_SNR
 
     # Remove snr rule from noise if needed
     if not target and 'snr' in rule:
@@ -602,7 +569,7 @@ def rand_augmentations(in_rules):
             for i in range(count):
                 new_rule = deepcopy(rule)
                 for key in new_rule:
-                    new_rule[key] = eval(re.sub(sonusai.mixture.rand_pattern, rand_repl, str(new_rule[key])))
+                    new_rule[key] = eval(re.sub(sonusai.mixture.RAND_PATTERN, rand_repl, str(new_rule[key])))
 
                     # convert eq values from strings to numbers
                     if key in ['eq1', 'eq2', 'eq3']:
@@ -615,18 +582,17 @@ def rand_augmentations(in_rules):
     return out_rules
 
 
-def get_base_name(config: dict, config_name: str) -> str:
+def get_output_from_config(config: dict, config_name: str) -> str:
     try:
         config_base = splitext(config_name)[0]
         name = str(splitext(config['output'])[0])
         name = name.replace('${config}', config_base)
         return name
     except Exception as e:
-        logger.exception(f'Error getting genmixdb base name: {e}')
-        raise SonusAIError
+        raise SonusAIError(f'Error getting genmixdb base name: {e}')
 
 
-def process_mixture(mixture: dict) -> dict:
+def _process_mixture(mixture: dict) -> dict:
     target_index = mixture['target_file_index']
     noise_offset = mixture['noise_offset']
     target_augmentation_index = mixture['target_augmentation_index']
@@ -635,8 +601,7 @@ def process_mixture(mixture: dict) -> dict:
 
     augmented_target_audio = apply_augmentation(audio_in=genmixdb_global['raw_target_audio'][target_index],
                                                 augmentation=target_augmentation,
-                                                length_common_denominator=genmixdb_global['feature_step_samples'],
-                                                dither=genmixdb_global['config']['dither'])
+                                                length_common_denominator=genmixdb_global['feature_step_samples'])
 
     noise_segment, _ = get_next_noise(offset_in=noise_offset,
                                       length=len(augmented_target_audio),
@@ -707,20 +672,19 @@ def main():
             start_time = time.monotonic()
             logger.info(f'Creating mixture database for {config_file}')
             config = get_config_from_file(config_file)
+            output = get_output_from_config(config, config_file)
 
-            base_name = get_base_name(config, config_file)
-            output_name = base_name + '.json'
-
-            log_name = base_name + '.log'
+            log_name = output + '.log'
             create_file_handler(log_name)
             update_console_handler(verbose)
             initial_log_messages('genmixdb')
 
             mixdb = genmixdb(config=config, show_progress=True)
 
-            with open(output_name, mode='w') as file:
+            json_name = output + '.json'
+            with open(file=json_name, mode='w') as file:
                 json.dump(mixdb, file, indent=2)
-                logger.info(f'Wrote mixture database for {config_file} to {output_name}')
+                logger.info(f'Wrote mixture database for {config_file} to {json_name}')
 
             end_time = time.monotonic()
             logger.info(f'Completed in {seconds_to_hms(seconds=end_time - start_time)}')
