@@ -1,4 +1,4 @@
-"""gentcst
+"""sonusai gentcst
 
 usage: gentcst [-hvru] [-o OUTPUT] [-f FOLD] [-y ONTOLOGY]
 
@@ -45,7 +45,9 @@ Outputs:
 """
 import csv
 import json
+import os
 import re
+from copy import deepcopy
 from os import getcwd
 from os.path import basename
 from os.path import dirname
@@ -63,8 +65,14 @@ from sonusai import create_file_handler
 from sonusai import initial_log_messages
 from sonusai import logger
 from sonusai import update_console_handler
+from sonusai.mixture import get_default_config
+from sonusai.mixture import load_config
+from sonusai.mixture import update_config_from_hierarchy
+from sonusai.mixture import update_truth_settings
 from sonusai.utils import trim_docstring
 
+CONFIG_FILE = 'config.yml'
+
 
 def gentcst(fold: str = '*',
             ontology: str = 'ontology.json',
@@ -84,11 +92,10 @@ def gentcst(fold: str = '*',
     logger.debug(f'update:       {update}')
     logger.debug('')
 
-    config = get_config('config.yml')
+    config = get_config()
 
     if config['truth_mode'] == 'mutex' and hierarchical:
-        logger.exception('Multi-class truth is incompatible with truth_mode mutex')
-        raise SonusAIError
+        raise SonusAIError('Multi-class truth is incompatible with truth_mode mutex')
 
     all_files = get_all_files(hierarchical)
     all_folds = get_folds_from_files(all_files)
@@ -113,31 +120,39 @@ def gentcst(fold: str = '*',
     config['targets'] = []
 
     logger.info(f'gentcst {len(use_files)} entries in tree')
+    root = os.getcwd()
     for file in use_files:
-        local_config = {}
-        local_name = dirname(file['file']) + '/config.yml'
-        if exists(local_name):
-            with open(local_name, mode='r') as yaml_file:
-                local_config = yaml.safe_load(yaml_file)
+        leaf = dirname(file['file'])
+        local_config = get_default_config()
+        local_config = update_config_from_hierarchy(root=root, leaf=leaf, config=local_config)
+        if not isinstance(local_config['truth_settings'], list):
+            local_config['truth_settings'] = [local_config['truth_settings']]
 
         specific_name = splitext(file['file'])[0] + '.yml'
         if exists(specific_name):
-            with open(specific_name, mode='r') as yaml_file:
-                specific_config = yaml.safe_load(yaml_file)
+            specific_config = load_config(specific_name)
 
             for key in specific_config:
-                local_config[key] = specific_config[key]
+                if key != 'truth_settings':
+                    local_config[key] = specific_config[key]
+                else:
+                    if not isinstance(specific_config['truth_settings'], list):
+                        specific_config['truth_settings'] = [specific_config['truth_settings']]
+                    update_truth_settings(truth_settings=specific_config['truth_settings'],
+                                          default=local_config['truth_settings'])
+                    local_config['truth_settings'] = specific_config['truth_settings']
 
-        if 'feature' in local_config.keys():
-            if config['feature'] != local_config['feature']:
-                logger.warning(
-                    f'Local feature for {file["file"]} ({local_config["feature"]}) is not the same as top level ({config["feature"]})')
+        truth_settings = deepcopy(local_config['truth_settings'])
+        for idx, val in enumerate(truth_settings):
+            val['index'] = file['truth_index']
+            for key in ['function', 'config']:
+                if key in val and val[key] == config['truth_settings'][idx][key]:
+                    del val[key]
 
-        target = {'target_name': file['file'], 'truth_index': file['truth_index']}
-        if 'truth_function' in local_config.keys():
-            target['truth_function'] = local_config['truth_function']
-        if 'truth_config' in local_config.keys():
-            target['truth_config'] = local_config['truth_config']
+        target = {
+            'target_name':    file['file'],
+            'truth_settings': truth_settings
+        }
 
         config['targets'].append(target)
 
@@ -151,7 +166,7 @@ def get_ontology(ontology: str) -> list:
     ontology_data = []
 
     if exists(ontology):
-        with open(ontology, encoding='utf-8') as f:
+        with open(file=ontology, mode='r', encoding='utf-8') as f:
             ontology_data = json.load(f)
 
     # Convert names to lowercase, convert spaces to '-', split into lists on ','
@@ -253,8 +268,7 @@ def get_name_from_id(ontology_data: list, identity: str) -> list:
     for item in ontology_data:
         if item['id'] == identity:
             if found:
-                logger.exception(f'id {identity} appears multiple times in ontology')
-                raise SonusAIError
+                raise SonusAIError(f'id {identity} appears multiple times in ontology')
 
             name = item['name']
             found = True
@@ -269,8 +283,7 @@ def get_id_from_name(ontology_data: list, name: str) -> str:
     for item in ontology_data:
         if name in item['name']:
             if found:
-                logger.exception(f'name {name} appears multiple times in ontology')
-                raise SonusAIError
+                raise SonusAIError(f'name {name} appears multiple times in ontology')
 
             identity = item['id']
             found = True
@@ -393,33 +406,23 @@ def get_index_from_label(labels: list, label: str) -> int:
     return next((item for item in labels if item['display_name'] == label), None)['index']
 
 
-def get_config(config_name: str) -> dict:
-    if not exists(config_name):
-        logger.exception('No confg.yml at top level')
-        raise SonusAIError
+def get_config() -> dict:
+    if not exists(CONFIG_FILE):
+        raise SonusAIError(f'No {CONFIG_FILE} at top level')
 
-    with open(config_name, mode='r') as yaml_file:
-        config = yaml.safe_load(yaml_file)
+    config = load_config(CONFIG_FILE)
 
-    if 'feature' not in config.keys():
-        logger.exception('feature not in top level config')
-        raise SonusAIError
+    if 'feature' not in config:
+        raise SonusAIError('feature not in top level config')
 
-    if 'truth_function' not in config.keys():
-        logger.exception('truth_function not in top level config')
-        raise SonusAIError
-
-    if 'truth_config' not in config.keys():
-        logger.exception('truth_config not in top level config')
-        raise SonusAIError
-
-    if 'truth_mode' not in config.keys():
-        logger.exception('truth_mode not in top level config')
-        raise SonusAIError
+    if 'truth_mode' not in config:
+        raise SonusAIError('truth_mode not in top level config')
 
     if config['truth_mode'] not in ['normal', 'mutex']:
-        logger.exception('Invalid truth_mode in top level config')
-        raise SonusAIError
+        raise SonusAIError('Invalid truth_mode in top level config')
+
+    if 'truth_settings' in config and not isinstance(config['truth_settings'], list):
+        config['truth_settings'] = [config['truth_settings']]
 
     return config
 
@@ -473,8 +476,8 @@ def write_metadata_to_tree(ontology_data: list, file: dict):
         if node:
             dir_name = dirname(file['file'])
             json_name = dir_name + '/' + basename(dir_name) + '.json'
-            with open(json_name, 'w') as json_file:
-                json.dump(node, json_file)
+            with open(file=json_name, mode='w') as f:
+                json.dump(node, f)
 
 
 def get_folds_from_leaf(all_files: list, leaf: str) -> list:
@@ -525,15 +528,15 @@ def main():
                                  update=update,
                                  verbose=verbose)
 
-        with open(output_name, 'w') as yaml_file:
-            yaml.dump(config, yaml_file)
+        with open(file=output_name, mode='w') as f:
+            yaml.dump(config, f)
             logger.info(f'Wrote config to {output_name}')
 
         csv_fields = ['index', 'display_name']
         csv_name = splitext(output_name)[0] + '.csv'
 
-        with open(csv_name, 'w') as csv_file:
-            writer = csv.DictWriter(csv_file, fieldnames=csv_fields)
+        with open(file=csv_name, mode='w') as f:
+            writer = csv.DictWriter(f, fieldnames=csv_fields)
             writer.writeheader()
             writer.writerows(labels)
             logger.info(f'Wrote labels to {csv_name}')
