@@ -1,42 +1,21 @@
-from os.path import expandvars
-
 import numpy as np
-import sox
-
-from sonusai import SonusAIError
-from sonusai import logger
-from sonusai import mixture
-
-
-def read_audio(name: str, dither: bool = True) -> np.ndarray:
-    expanded_name = expandvars(name)
-
-    try:
-        # Read in and convert to desired format
-        inp = sox.Transformer()
-        inp.set_output_format(rate=mixture.sample_rate, bits=mixture.bit_depth, channels=mixture.channel_count)
-        inp.set_globals(dither=dither)
-        return inp.build_array(input_filepath=expanded_name,
-                               sample_rate_in=int(sox.file_info.sample_rate(expanded_name)))
-
-    except Exception as e:
-        if name != expanded_name:
-            logger.exception(f'Error reading {name} (expanded: {expanded_name}): {e}')
-        else:
-            logger.exception(f'Error reading {name}: {e}')
-        raise SonusAIError
 
 
 def apply_augmentation(audio_in: np.ndarray,
                        augmentation: dict,
-                       length_common_denominator: int,
-                       dither: bool = True) -> np.ndarray:
+                       length_common_denominator: int) -> np.ndarray:
+    import sox
+
+    from sonusai import SonusAIError
+    from sonusai.mixture.constants import BIT_DEPTH
+    from sonusai.mixture.constants import CHANNEL_COUNT
+    from sonusai.mixture.constants import SAMPLE_RATE
+
     try:
         # Apply augmentations
         tfm = sox.Transformer()
-        tfm.set_input_format(rate=mixture.sample_rate, bits=mixture.bit_depth, channels=mixture.channel_count)
-        tfm.set_output_format(rate=mixture.sample_rate, bits=mixture.bit_depth, channels=mixture.channel_count)
-        tfm.set_globals(dither=dither)
+        tfm.set_input_format(rate=SAMPLE_RATE, bits=BIT_DEPTH, channels=CHANNEL_COUNT)
+        tfm.set_output_format(rate=SAMPLE_RATE, bits=BIT_DEPTH, channels=CHANNEL_COUNT)
 
         # TODO
         #  Always normalize and remove normalize from list of available augmentations
@@ -74,7 +53,7 @@ def apply_augmentation(audio_in: np.ndarray,
             tfm.lowpass(frequency=augmentation['lpf'])
 
         # Create output data
-        audio_out = tfm.build_array(input_array=audio_in, sample_rate_in=mixture.sample_rate)
+        audio_out = tfm.build_array(input_array=audio_in, sample_rate_in=SAMPLE_RATE)
 
         # make sure length is multiple of length_common_denominator
         audio_len = len(audio_out)
@@ -84,5 +63,4 @@ def apply_augmentation(audio_in: np.ndarray,
 
         return audio_out
     except Exception as e:
-        logger.exception(f'Error applying {augmentation}: {e}')
-        raise SonusAIError
+        raise SonusAIError(f'Error applying {augmentation}: {e}')
