@@ -1,7 +1,4 @@
 import numpy as np
-from pyaaware import ForwardTransform
-
-from sonusai.utils import int16_to_float
 
 
 def generate_segsnr(mixdb: dict,
@@ -10,6 +7,10 @@ def generate_segsnr(mixdb: dict,
                     noise: np.ndarray,
                     compute: bool = True,
                     frame_based: bool = False) -> np.ndarray:
+    from pyaaware import ForwardTransform
+
+    from sonusai.utils import int16_to_float
+
     if not compute:
         return np.empty(0, dtype=np.single)
 
@@ -23,12 +24,19 @@ def generate_segsnr(mixdb: dict,
     frame = 0
     for offset in range(0, record['samples'], mixdb['frame_size']):
         indices = slice(offset, offset + mixdb['frame_size'])
+
         target_energy = fft.energy(int16_to_float(target[indices]))
         noise_energy = fft.energy(int16_to_float(noise[indices]))
+
+        if noise_energy == 0:
+            snr = np.single(np.inf)
+        else:
+            snr = np.single(target_energy / noise_energy)
+
         if frame_based:
-            segsnr[frame] = np.single(target_energy / noise_energy)
+            segsnr[frame] = snr
             frame += 1
         else:
-            segsnr[indices] = np.single(target_energy / noise_energy)
+            segsnr[indices] = snr
 
     return segsnr
