@@ -1,15 +1,7 @@
-from copy import deepcopy
+from typing import List
+from typing import Union
 
-import h5py
 import numpy as np
-from pyaaware import ForwardTransform
-from pyaaware import SED
-
-from sonusai import SonusAIError
-from sonusai import logger
-from sonusai.mixture import get_class_count
-from sonusai.mixture import get_class_weights_threshold
-from sonusai.utils import int16_to_float
 
 
 def strictly_decreasing(list_to_check: list) -> bool:
@@ -21,31 +13,27 @@ def generate_truth(mixdb: dict,
                    target: np.ndarray,
                    noise: np.ndarray,
                    compute: bool = True) -> np.ndarray:
+    from copy import deepcopy
+
+    from sonusai.mixture.class_count import get_class_count
+    from sonusai.mixture.get_class_weights_threshold import get_class_weights_threshold
+
     if not compute:
         return np.empty(0, dtype=np.single)
 
-    truth_config = deepcopy(mixdb['targets'][record['target_file_index']]['truth_config'])
-    truth_config['frame_size'] = mixdb['frame_size']
-    truth_config['num_classes'] = mixdb['num_classes']
-    truth_config['mutex'] = mixdb['truth_mutex']
-    truth_config['target_gain'] = record['target_gain']
-
-    indices = mixdb['targets'][record['target_file_index']]['truth_index']
-    functions = mixdb['targets'][record['target_file_index']]['truth_function']
-
-    if len(functions) != len(indices):
-        logger.exception('length of truth_index does not match length of truth_function')
-        raise SonusAIError
-
-    truth = np.zeros((len(target), truth_config['num_classes']), dtype=np.single)
-    for i in range(len(functions)):
-        truth_config['function'] = functions[i]
-        truth_config['index'] = indices[i]
+    truth = np.zeros((len(target), mixdb['num_classes']), dtype=np.single)
+    for truth_setting in mixdb['targets'][record['target_file_index']]['truth_settings']:
+        config = deepcopy(truth_setting)
+        config['frame_size'] = mixdb['frame_size']
+        config['num_classes'] = mixdb['num_classes']
+        config['mutex'] = mixdb['truth_mutex']
+        config['target_gain'] = record['target_gain']
         new_truth = truth_function(target=target,
                                    noise=noise,
-                                   config=truth_config)
+                                   config=config)
         truth = truth + new_truth
 
+    indices = [sub['index'] for sub in mixdb['targets'][record['target_file_index']]['truth_settings']]
     record['class_count'] = get_class_count(
         truth_index=indices,
         truth=truth,
@@ -57,24 +45,31 @@ def generate_truth(mixdb: dict,
 def truth_function(target: np.ndarray,
                    noise: np.ndarray,
                    config: dict) -> np.ndarray:
+    import h5py
+    from pyaaware import ForwardTransform
+    from pyaaware import SED
+
+    from sonusai import SonusAIError
+    from sonusai.utils import int16_to_float
+
     if config['function'] == 'sed':
         if config['target_gain'] == 0:
             return np.zeros((len(target), config['num_classes']), dtype=np.single)
 
         if len(target) % config['frame_size'] != 0:
-            logger.exception(f'Number of samples in audio is not a multiple of {config["frame_size"]}')
-            raise SonusAIError
+            raise SonusAIError(f'Number of samples in audio is not a multiple of {config["frame_size"]}')
 
-        if 'thresholds' in config.keys():
-            thresholds = config['thresholds']
-            if not isinstance(thresholds, list) or len(thresholds) != 3:
-                logger.exception(f'Truth function SED thresholds does not contain 3 entries: {thresholds}')
-                raise SonusAIError
-            if not strictly_decreasing(thresholds):
-                logger.exception(f'Truth function SED thresholds are not strictly decreasing: {thresholds}')
-                raise SonusAIError
-        else:
-            thresholds = None
+        if 'config' not in config:
+            raise SonusAIError('Truth function SED missing config')
+
+        parameters = ['thresholds']
+        for parameter in parameters:
+            if parameter not in config['config']:
+                raise SonusAIError(f'Truth function SED config missing required parameter: {parameter}')
+
+        thresholds = config['config']['thresholds']
+        if not strictly_decreasing(thresholds):
+            raise SonusAIError(f'Truth function SED thresholds are not strictly decreasing: {thresholds}')
 
         fft = ForwardTransform(N=config['frame_size'] * 4, R=config['frame_size'])
         sed = SED(thresholds=thresholds,
@@ -93,22 +88,22 @@ def truth_function(target: np.ndarray,
         return truth
 
     elif config['function'] == 'file':
-        file_parameters = ['file']
-        for parameter in file_parameters:
-            if parameter not in config.keys():
-                logger.exception(f'Truth function config missing required parameter: {parameter}')
-                raise SonusAIError
+        if 'config' not in config:
+            raise SonusAIError('Truth function file missing config')
 
-        with h5py.File(name=config['file'], mode='r') as f:
+        parameters = ['file']
+        for parameter in parameters:
+            if parameter not in config['config']:
+                raise SonusAIError(f'Truth function file config missing required parameter: {parameter}')
+
+        with h5py.File(name=config['config']['file'], mode='r') as f:
             truth_in = f['/truth_t'][:]
 
         if truth_in.ndim != 2:
-            logger.exception('Truth file data is not 2 dimensions')
-            raise SonusAIError
+            raise SonusAIError('Truth file data is not 2 dimensions')
 
         if truth_in.shape[0] != len(target):
-            logger.exception('Truth file does not contain the right amount of samples')
-            raise SonusAIError
+            raise SonusAIError('Truth file does not contain the right amount of samples')
 
         truth = np.zeros((len(target), config['num_classes']), dtype=np.single)
         if config['target_gain'] == 0:
@@ -116,14 +111,12 @@ def truth_function(target: np.ndarray,
 
         if isinstance(config['index'], list):
             if len(config['index']) != truth_in.shape[1]:
-                print('Truth file does not contain the right amount of classes')
-                raise SonusAIError
+                raise SonusAIError('Truth file does not contain the right amount of classes')
 
             truth[:, config['index']] = truth_in
         else:
             if config['index'] + truth_in.shape[1] > config['num_classes']:
-                print('Truth file contains too many classes')
-                raise SonusAIError
+                raise SonusAIError('Truth file contains too many classes')
 
             truth[:, config['index']:config['index'] + truth_in.shape[1]] = truth_in
 
@@ -143,8 +136,25 @@ def truth_function(target: np.ndarray,
         # Read in .txt transcript and run a Python function to generate text grid data
         # (indicating which phonemes are active)
         # Then generate truth based on this data and put in the correct classes based on config['index']
-        logger.exception('phoneme function is not supported yet')
-        raise SonusAIError
+        raise SonusAIError('Truth function phoneme is not supported yet')
 
-    logger.exception(f'Unsupported truth function: {config["function"]}')
-    raise SonusAIError
+    raise SonusAIError(f'Unsupported truth function: {config["function"]}')
+
+
+def get_truth_indices(mixdb: dict, mixid: Union[str, List[int]]) -> list:
+    from sonusai.mixture.get_mixtures_from_mixid import convert_mixid_to_list
+
+    indices = list()
+
+    for m in convert_mixid_to_list(mixdb, mixid):
+        target_file_index = mixdb['mixtures'][m]['target_file_index']
+
+        # extract indices from truth_settings
+        index = [sub['index'] for sub in mixdb['targets'][target_file_index]['truth_settings']]
+
+        # flatten, uniquify, and sort
+        index = sorted(list(set([item for sublist in index for item in sublist])))
+
+        indices.append(index)
+
+    return indices
