@@ -1,23 +1,19 @@
-from sonusai import SonusAIError
-from sonusai import logger
-
-
 def get_offsets(mixdb: dict, mixid: int) -> (int, int, int):
+    from sonusai import SonusAIError
+
     required_keys = [
         'mixtures',
         'frame_size',
         'feature_step_samples',
     ]
     for key in required_keys:
-        if key not in mixdb.keys():
-            logger.exception(f'Missing {key} in mixdb')
-            raise SonusAIError
+        if key not in mixdb:
+            raise SonusAIError(f'Missing {key} in mixdb')
 
     if mixid >= len(mixdb['mixtures']) or mixid < 0:
-        logger.exception(f'Invalid mixid: {mixid}')
-        raise SonusAIError
+        raise SonusAIError(f'Invalid mixid: {mixid}')
 
-    i_sample_offset = sum([sub['samples'] for sub in mixdb['mixtures'][:mixid + 1]])
+    i_sample_offset = sum([sub['samples'] for sub in mixdb['mixtures'][:mixid]])
     i_frame_offset = i_sample_offset // mixdb['frame_size']
     o_frame_offset = i_sample_offset // mixdb['feature_step_samples']
 
