@@ -1,17 +1,21 @@
-import h5py
+from typing import Union
+
 import numpy as np
 
-from sonusai import logger
 
+def read_predict_data(filename: str, expected_frames: int) -> Union[None, np.ndarray]:
+    """Read predict data from given HDF5 file and return it."""
+    import h5py
+
+    from sonusai import logger
 
-def read_predict_data(filename: str, expected_frames: int) -> (None, np.ndarray):
     if not filename:
         return None
 
     logger.info(f'Reading prediction data from {filename}')
     with h5py.File(name=filename, mode='r') as f:
         # prediction data is either [frames, num_classes], or [frames, timesteps, num_classes]
-        predict = f['predict'][:]
+        predict = np.array(f['predict'])
 
         if predict.ndim == 2:
             frames, num_classes = predict.shape
